#!/usr/bin/perl
#
# Copyright (c) 2004-2005 Mathieu Roy <yeupou--gnu.org>
# http://yeupou.coleumes.org
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
#   USA
#
# $Id: integcheck-shell.pl,v 1.13 2005/09/23 15:04:08 yeupou Exp $
#
# Part of integcheck, http://gna.org/projects/integcheck

use strict;
use Digest::MD5 qw(md5_base64);
use File::Find::Rule;

our @checklist = ("/bin",
		  "/sbin",
		  "/lib",
		  "/boot",
		  "/etc",
		  "/usr/bin", 
		  "/usr/sbin",
		  "/usr/lib",  
		  "/usr/local/bin",
		  "/usr/local/sbin",
		  "/usr/local/lib");

if (-e "/etc/checklist") {
    # We run the file, but from a security point of view, it may be better
    # to have a read-only file parsed.
    # If you want to fix that, go ahead.
    # This file must be writable only by root and executable-readable only
    # for the integcheck user.
    do "/etc/checklist"
	or print "Unable to run /etc/checklist\n";
}

# check for subdirs
# We only build a list of directories, not a list of files, in order to save
# memory (otherwise we would save list of thousands files). 
my @final_checklist;
foreach my $dir (@checklist) {
    my @subdirectories = File::Find::Rule->directory()
	->directory
	->in($dir);
    push(@final_checklist, @subdirectories);
}

# go
print "#### Start HERE ####\n";
    
foreach my $dir (sort(@final_checklist)) {
    if (-e $dir) {
	if (-d $dir) {
	    opendir(REP, $dir);
	    while (defined(my $file = readdir(REP))) {
		# Accept only files
		next unless -f "$dir/$file"; 
		# Ignore backups
		next if $file =~ /~$/;
		# Open file, if possible
		open(FILE, "$dir/$file") or next;
		binmode(FILE);
		# Print md5 base64
		print "$dir/$file:".Digest::MD5->new->addfile(*FILE)->b64digest.":".(stat("$dir/$file"))[7]."\n";
		close(FILE);
	    }
	    closedir(REP);
	}
    } else {
	print "$dir not found!\n";
    }
}

print "#### Finish HERE ####\n";

exit;

# EOF
